@@ -2,17 +2,16 @@ from typing import Callable, Any, List
 
 import networkx as nx
 
-from .types import Variable, Function, Data, GraphDict
+from .types import Variable, Function, Data, GraphDict, GraphObject
 from .draw import draw_compute_graph
-from .dynamic import split_static_dynamic
-from .utils import expand_nested_dict, get_with_injected_parameters, get_input_variables
+from .utils import trace_with_named_keys, trace_object, get_input_variables, filter_graph, query
 
 """
 Graph building functions
 """
 
 
-def build_dag(graph_dict: GraphDict, local_source_name="graph_locals") -> nx.DiGraph:
+def build_dag(graph_dict: dict, local_source_name="graph_locals") -> nx.DiGraph:
     """Build a DiGraph from the supplied GraphDict
 
     Args:
@@ -23,8 +22,10 @@ def build_dag(graph_dict: GraphDict, local_source_name="graph_locals") -> nx.DiG
     """
     dag = nx.digraph.DiGraph()
     for name, node_spec in graph_dict.items():
-        if not (isinstance(node_spec, Function) or isinstance(node_spec, Data)):
-            raise TypeError("Invalid node type, expected Function or Data", node_spec)
+        if not (isinstance(node_spec, GraphObject)):
+            raise TypeError(
+                "Invalid node type, expected Function, Variable, or Data", type(node_spec)
+            )
         dag.add_node(name, node_spec=node_spec)
     trace_edges(dag, local_source_name)
     return dag
@@ -38,19 +39,11 @@ def trace_edges(dag: nx.DiGraph, local_source_name="graph_locals"):
     """
 
     def find_source_node(dag: nx.DiGraph, var: Variable):
-        source_key = var.name
+        source_key = var.key
         source_node = None
         if source_key in dag:
             source_node = source_key
         else:
-            split_k = source_key.split(".")
-            for i in range(1, len(split_k)):
-                # Find at a higher layer that does exist
-                layer_k = ".".join(split_k[:-i])
-                if layer_k in dag:
-                    source_node = layer_k
-                    break
-        if source_node is None:
             raise KeyError(source_key)
         return source_node
 
@@ -62,7 +55,7 @@ def trace_edges(dag: nx.DiGraph, local_source_name="graph_locals"):
                     source_node = find_source_node(dag, a)
                     dag.add_edge(source_node, node)
             for k, v in node_spec.kwargs.items():
-                if isinstance(a, Variable) and (a.source == local_source_name):
+                if isinstance(v, Variable) and (v.source == local_source_name):
                     # We have a nested key
                     source_node = find_source_node(dag, v)
                     dag.add_edge(source_node, node)
@@ -75,10 +68,9 @@ Operations over built DiGraphs
 
 def build_callable(
     dag: nx.DiGraph,
-    local_source_name="graph_locals",
-    nested_params=True,
+    targets=None,
     include_inputs=False,
-    out_keys=None,
+    local_source_name="graph_locals",
 ) -> Callable[[dict], Any]:
     """Returns a callable Python function corresponding to this graph
 
@@ -94,75 +86,109 @@ def build_callable(
 
     node_dict = {node: dag.nodes[node]["node_spec"] for node in ggen}
 
-    def compute_from_params(**kwargs):
+    def compute_from_params(**sources):
         out_p = {}
 
-        if nested_params:
-            sources = {}
-            for k, v in kwargs.items():
-                sources[k] = expand_nested_dict(v, include_parents=True)
-        else:
-            sources = kwargs.copy()
+        # sources = kwargs.copy()
+
+        sources[local_source_name] = out_p
 
         for node, node_spec in node_dict.items():  # ggen:
-            # node_spec = dag.nodes[node]["node_spec"]
-            if isinstance(node_spec, Variable):
-                out_p[node] = sources[node_spec.source][node_spec.name]
-            elif isinstance(node_spec, Function):
-                sources.update({local_source_name: out_p})
-                out_p[node] = node_spec.call(sources)
-            elif isinstance(node_spec, Data):
-                out_p[node] = node_spec.data
-            else:
-                raise Exception("Unsupported node type", node, node_spec, type(node_spec))
+            out_p[node] = node_spec.evaluate(**sources)
 
         if include_inputs:
             out_p.update({k: v for k, v in sources.items() if k != "graph_locals"})
 
         return out_p
 
-    if out_keys is None:
+    if targets is None:
         return compute_from_params
     else:
 
         def compute_for_keys(**kwargs):
             results = compute_from_params(**kwargs)
-            return {k: results[k] for k in out_keys}
+            return {k: results[k] for k in targets}
 
         return compute_for_keys
 
 
-"""
-Object Oriented Wrappers
-"""
-
-
 class ComputeGraph:
     """A thin object oriented wrapper around the graph management functions"""
 
-    def __init__(self, graph_dict: dict, local_source_name="graph_locals"):
+    def __init__(
+        self,
+        graph_dict: dict,
+        local_source_name="graph_locals",
+        is_traced=False,
+        targets=None,
+        validate_keys=True,
+    ):
         """Build a fully traced DiGraph from the supplied dict
 
         Args:
             graph_dict: A dict with keys as node names, and arguments of
         """
+        if isinstance(graph_dict, GraphObject):
+            self._targets = ["out"]
+            graph_dict, _ = trace_with_named_keys({"out": graph_dict}, validate_keys)
+        elif not is_traced:
+            self._targets = list(graph_dict.keys())
+            graph_dict, _ = trace_with_named_keys(graph_dict, validate_keys)
+        else:
+            self._targets = []
+
+        if targets is not None:
+            self._targets = targets
+
         self.dag = build_dag(graph_dict, local_source_name)
         self.local_source_name = local_source_name
-        self.pdag = get_with_injected_parameters(self.dag)
         self.dict = graph_dict
 
     def draw(self, targets=None, **kwargs):
-        return draw_compute_graph(self.pdag, targets, **kwargs)
+        if targets is None:
+            targets = self._targets
+        return draw_compute_graph(self.dag, targets, **kwargs)
 
-    def freeze(self, dynamic_inputs: List[Variable], targets: List[str], inputs: dict):
-        return split_static_dynamic(self.dict, dynamic_inputs, targets, **inputs)
+    def freeze(
+        self,
+        dynamic_inputs: List[Variable],
+        input_variables: dict = None,
+        targets: List[str] = None,
+    ):
+        from .dynamic import freeze_graph
+
+        if targets is None:
+            targets = self._targets
+
+        return freeze_graph(self, targets, dynamic_inputs, input_variables)
+
+    def filter(self, targets=None, sources=None, exclude=None):
+        if targets is None and sources is None:
+            targets = self._targets
+        return filter_graph(self, targets, sources, exclude)
+
+    def query(self, pattern: str) -> List[str]:
+        """Return a list of keys from this graph which match the supplied regex pattern
+
+        Args:
+            pattern: Regex pattern
+
+        Returns:
+            List[str]: List of matching keys
+        """
+        return query(self, pattern)
 
     def get_input_variables(self):
         return get_input_variables(self.dag)
 
     def get_callable(
-        self, nested_params=True, include_inputs=False, out_keys=None
+        self, targets=None, include_inputs=False, output_all=False
     ) -> Callable[[dict], Any]:
-        return build_callable(
-            self.dag, self.local_source_name, nested_params, include_inputs, out_keys
-        )
+        if output_all:
+            if targets:
+                raise Exception("Target list supplied yet all outputs requested")
+        else:
+            if targets is None:
+                targets = self._targets
+
+        return build_callable(self.dag, targets, include_inputs, self.local_source_name)
